#!/usr/bin/Rscript --vanilla

#-------------------------------------------------------------------------------
# Aim: correcting the raw export from ODK for FindPhoto data
# Author (aut): Néhémie Strupler / adapted by Toby C. Wilkinson
# Maintainer (cre): Néhémie Strupler, Toby Wilkinson
# Last modified: 2020-06-26
# Licence: CC BY-SA 4.0
#-------------------------------------------------------------------------------

# Load the data
## Define fixed variable

wd_survey <- c("../../survey-data/")
photo_folder <- c("photo-data/")
findphoto_folder_odk <- paste0(photo_folder, "export-odk/")
findphoto_folder_canonical <- paste0(photo_folder, "export-odk/canonical/")
findphoto_r1  <- "2019/FindPhoto_r1/FindPhoto_r1.csv"
findphoto_r2  <- "2019/FindPhoto_r2/FindPhoto_r2.csv"
findphoto_photos_r1  <- "2019/FindPhoto_r1/FindPhoto_r1-photos.csv"
findphoto_photos_r2  <- "2019/FindPhoto_r2/FindPhoto_r2-photos.csv"
findphoto_canonical <- "FindPhoto_summary_data.csv"


description_findphoto_data_path <- paste0(wd_survey, photo_folder,
                                     "export-odk/description_canonical_findphoto-data.csv")

### Load the poi data
findphoto_data_r1 <-
  read.csv(paste0(wd_survey, findphoto_folder_odk, findphoto_r1), encoding = "UTF-8")
findphoto_data_r2 <-
  read.csv(paste0(wd_survey, findphoto_folder_odk, findphoto_r2), encoding = "UTF-8")

findphoto_photos_data_r1 <-
  read.csv(paste0(wd_survey, findphoto_folder_odk, findphoto_photos_r1), encoding = "UTF-8")
findphoto_photos_data_r2 <-
  read.csv(paste0(wd_survey, findphoto_folder_odk, findphoto_photos_r2), encoding = "UTF-8")

description_findphoto_data <-
  read.csv(description_findphoto_data_path,
           encoding = "UTF-8", na.strings = c(NA,""))

findphoto_data_r1$revision <- "FindPhoto_r1"
findphoto_data_r1$`SET.OF.photos` <- NULL
findphoto_data_r2$revision <- "FindPhoto_r2"
findphoto_data_r2$`SET.OF.photos` <- NULL

findphoto_data <- rbind(findphoto_data_r1, findphoto_data_r2)
rm(findphoto_data_r1, findphoto_data_r2)

findphoto_photos_data <- rbind(findphoto_photos_data_r1,findphoto_photos_data_r2)
rm(findphoto_photos_data_r1,findphoto_photos_data_r2)

### Merge or join data and file data

# combine photo_data with data_canonical
findphoto_data <- merge(findphoto_photos_data, findphoto_data,
                          by.x = "PARENT_KEY", by.y = "KEY", all.x = TRUE)
rm(findphoto_photos_data)

### Remove items where there is no linking media
findphoto_data <- findphoto_data[!is.na(findphoto_data$find_photo) & !findphoto_data$find_photo=="",]

### Standardise datefields
library(lubridate)

findphoto_data$today <- format(lubridate::parse_date_time(findphoto_data$today, orders=c("d-b-Y","b d, Y")),"%d-%b-%Y")
findphoto_data$start <- format(lubridate::parse_date_time(findphoto_data$start, orders=c("d-b-Y H:M:S","b d, Y I:M:S p")),"%H:%M:%S %z")
findphoto_data$end <- format(lubridate::parse_date_time(findphoto_data$end, orders=c("d-b-Y H:M:S","b d, Y I:M:S p")),"%H:%M:%S %z")

### Create a longer filename so that image files can be identified in photo-archive
# tract_photo and revision should be merged so that filename reflects source and photo archive
findphoto_data$ShortFileName <- findphoto_data$find_photo
findphoto_data$find_photo <- paste("",format(as.Date(findphoto_data$today, format="%d-%b-%Y"),"%Y"),"Core","Finds","FindPhoto_ODK",findphoto_data$revision,findphoto_data$find_photo,sep="/")

### Order by TractNo
findphoto_data <- findphoto_data[order(findphoto_data$context_data.tract_no), ]
rownames(findphoto_data) <- 1:nrow(findphoto_data)

## Change variable names with file description
findphoto_data <- subset(findphoto_data,
                      select = as.character(description_findphoto_data$ODKExportName))
colnames(findphoto_data) <- description_findphoto_data$Variable
findphoto_data <- findphoto_data[!is.na(colnames(findphoto_data))]
findphoto_data[is.na(findphoto_data)] <- ""

# Remove unwanted columns
findphoto_data$`_NOT_EXPORTED_TO_CANONICAL_` <- NULL

########################################################
### Make corrections to individual records
####### 2019

## Tr 3166 can't be right. It must be 6166 and w038 did indeed have diagnostics from 6166 (note that the Find data also needs updating)
tocorrect <- c('uuid:86298221-e4bb-433b-aaf8-48a818810215')
findphoto_data[findphoto_data$ODK_PARENT_KEY %in% tocorrect,]$TractNo <- 6166

## Tr 5980 was not walker (at least not in 2019), this must be 3980
tocorrect <- c('uuid:a5d0a9d3-ecca-4563-9c6d-aaef2e8f70b8')
findphoto_data[findphoto_data$ODK_PARENT_KEY %in% tocorrect,]$TractNo <- 3980

## Tr 3978 w003 can't be right. If Tract is correct, w038 is the only who collected diagnostics
tocorrect <- c("uuid:0a84aede-0b00-4366-8e53-7cb61ccd4b70",
  "uuid:43b3e25e-aef1-46a1-85dd-12e2acc38951",
  "uuid:5ae8ef30-fc9b-4997-8da4-005cfd4d53b3",
  "uuid:df5ca76e-2bd0-4c2e-b0c3-856b396982c8")
findphoto_data[findphoto_data$ODK_PARENT_KEY %in% tocorrect,]$WalkerNo <- "w038"

########################################################
#### Finished corrections

#### Make a copy of data for comparison purposes
findphoto_data_test <- findphoto_data

for (i in 1:ncol(findphoto_data_test)) {
  if (is.factor(findphoto_data_test[,i]) == TRUE) {
    findphoto_data_test[,i] <- as.character(findphoto_data_test[,i])
  } else
  if (is.integer(findphoto_data_test[,i]) == TRUE) {
    findphoto_data_test[,i] <- as.numeric(findphoto_data_test[,i])
  }
}

# Read canonical data and make them similar to poi data
findphoto_data_initial_path <- paste0(wd_survey, findphoto_folder_canonical, findphoto_canonical)

if (file.exists(findphoto_data_initial_path)) {
  findphoto_data_initial <- read.csv(findphoto_data_initial_path, stringsAsFactors = FALSE)
  for (i in 1:ncol(findphoto_data_initial)) {
    if (is.integer(findphoto_data_initial[,i]) == TRUE) {
      findphoto_data_initial[,i] <- as.numeric(findphoto_data_initial[,i])
    }
  }
  findphoto_data_initial[is.na(findphoto_data_initial)] <- ""

  test_diff <- identical(findphoto_data_test, findphoto_data_initial)
  rm(findphoto_data_test, findphoto_data_initial)



  # Exporting data if needed and user say Y
  if (test_diff == FALSE) {
    while (test_diff != "Y" | test_diff != "N") {
      cat("\n \n The file 'findphoto_data' has been changed and")
      cat("\n the canonical version will be updated.")
      cat("\n Do you agree [Y/N]? ")
      cat("\n (use a capital letter)   :")
      test_diff  <- readChar("stdin", 1)
      if (test_diff == "Y") {
        write.csv(findphoto_data,
                  file = paste(wd_survey,findphoto_folder_canonical,
                               findphoto_canonical, sep = ""),
                  row.names = FALSE)
        cat("\n Canonical data have been overwritten\n \n")
        break}  # breaks when hit enter
      if (test_diff == "N") {
        cat("\n Canonical data have NOT been overwritten\n \n")
        break}  # breaks when hit enter
    }
  } else
    cat("\n findphoto_data are already identical to canonical data \n \n ")
}
if (!file.exists(findphoto_data_initial_path)) {
  write.csv(findphoto_data,
          file = findphoto_data_initial_path,
          row.names = FALSE)
  cat("\n a file findphoto_data has been added in the folder canonical \n \n ")
}
