#!/usr/bin/Rscript --vanilla

#--------------------------------------------------------------------------------
# Aim: correcting the raw export from ODK for photos
# Author (aut): Néhémie Strupler / adapted by Toby C. Wilkinson
# Maintainer (cre): Néhémie Strupler / Toby C. Wilkinson
# Last modified: 2017-08-19
# Licence: CC BY 4.0
#--------------------------------------------------------------------------------

# Load the data ------------------------------------------
## Define fixed variable

wd_survey <- c("../../survey-data/")
tract_folder <- c("tract-data/")
tract_folder_odk <- paste(tract_folder, "export_odk/", sep = "")
tract_folder_canonical <- paste(tract_folder, "canonical/", sep = "")
tract_r2_photos  <- "Tract_r2/Tract_photos.csv"
tract_r4_photos  <- "Tract_r4/Tract_r4_photos.csv"
tract_r7_photos  <- "Tract_r7/Tract_r7_photos.csv"
tract_r10_photos  <- "Tract_r10/Tract_r10-photos.csv"
tract_r11_photos  <- "Tract_r11/Tract_r11-photos.csv"
tract_canonical <- "Tract_general_data.csv"
tract_canonical_photo <- "Tract_photo_data.csv"
description_tract_photo_data_path <- paste(wd_survey, tract_folder,
                                     "description_canonical_tract-photo-data.csv", sep = "")

### Load the tract data
tract_data_r2_photos <-
  read.csv(paste(wd_survey, tract_folder_odk, tract_r2_photos, sep = ""),
          encoding = "UTF-8")

tract_data_r4_photos <-
  read.csv(paste(wd_survey, tract_folder_odk, tract_r4_photos, sep = ""),
              encoding = "UTF-8")

tract_data_r7_photos <-
  read.csv(paste(wd_survey, tract_folder_odk, tract_r7_photos, sep = ""),
              encoding = "UTF-8")

tract_data_r10_photos <-
  read.csv(paste(wd_survey, tract_folder_odk, tract_r10_photos, sep = ""),
              encoding = "UTF-8")

tract_data_r11_photos <-
  read.csv(paste(wd_survey, tract_folder_odk, tract_r11_photos, sep = ""),
              encoding = "UTF-8")

tract_data_canonical <-
                read.csv(paste(wd_survey, tract_folder_canonical, tract_canonical, sep = ""),
                            encoding = "UTF-8")

description_tract_photo_data <-
  read.csv(description_tract_photo_data_path,
           encoding = "UTF-8", na.strings = c(NA,""))

# Combining r2 and r4
### ABOUT R2 AND R4: there were no changes in the format of the _photos file
### HOWEVER: for the organisation of photographs, an additional field needs to be added to acknowledge
### revision version of form

tract_data_r2_photos$revision <- "Tract_r2"
tract_data_r4_photos$revision <- "Tract_r4"
tract_data_r7_photos$revision <- "Tract_r7"
tract_data_r10_photos$revision <- "Tract_r10"
tract_data_r11_photos$revision <- "Tract_r11"
#r7 for some reasons misses the media/ listing from the export of photo filename
tract_data_r7_photos$tract_photo <- paste0("media/",tract_data_r7_photos$tract_photo)
tract_data_r10_photos$tract_photo <- paste0("media/",tract_data_r10_photos$tract_photo)
tract_data_r11_photos$tract_photo <- paste0("media/",tract_data_r11_photos$tract_photo)

tract_photo_data <- rbind(tract_data_r2_photos, tract_data_r4_photos)
tract_photo_data <- rbind(tract_photo_data, tract_data_r7_photos)
tract_photo_data <- rbind(tract_photo_data, tract_data_r10_photos)
tract_photo_data <- rbind(tract_photo_data, tract_data_r11_photos)

rm(tract_data_r2_photos, tract_data_r4_photos, tract_data_r7_photos)

## Correction

# [Ph] with PARENT_KEY=uuid:0db9924c-fb17-4398-a550-35475c8db5e0
# (nw. 5377) replace with in canonical
# PARENT_KEY=uuid:15600cef-e6ec-4830-a30d-33fcdf0138d4 (i.e. nw. 3791)
tract_photo_data[tract_photo_data$PARENT_KEY == "uuid:0db9924c-fb17-4398-a550-35475c8db5e0",
                 "PARENT_KEY"] <- "uuid:15600cef-e6ec-4830-a30d-33fcdf0138d4"

### Remove photos without tract_photo i.e. FileName (seeing as how they don't
### actually relate to anything)
### These are probably fake data introduced by a break in connection between collection device
### and the ODK Aggregate instance where they were aggregated.
### However it is also possible that some images are unlinked.

cat("\n Any entries with no tract_photo value will be listed here \n \n ")

tract_photo_data[is.na(tract_photo_data$tract_photo),]

tract_photo_data[tract_photo_data$tract_photo == "",]

# remove these problems from the data
tract_photo_data <- tract_photo_data[!is.na(tract_photo_data$tract_photo),]
tract_photo_data <- tract_photo_data[!tract_photo_data$tract_photo == "",]

### Merge or join tract_data_canonical and tract_photo_data to find TractID

# Just extract the wanted columns from canonical (i.e. cleaned and corrected) version
tract_data_canonical <- subset(tract_data_canonical, select=c("TractNo", "ODK_KEY", "FormToday"))

# combine tract_photo_data with tract_data_canonical
tract_photo_data <- merge(x = tract_photo_data, y = tract_data_canonical,
    by.x = "PARENT_KEY", by.y = "ODK_KEY", all.x = TRUE)

### Test for missing TractIDs (i.e. non matching IDs); list them and print them

cat("\n Any photos which cannot be linked with TractNo will be listed here \n \n ")

tract_photo_data[is.na(tract_photo_data$TractNo),]

### Remove photos without identifiable link to TractID

tract_photo_data <- tract_photo_data[!is.na(tract_photo_data$TractNo),]

### Create a longer filename so that image files can be identified in photo-archive
# tract_photo and revision should be merged so that filename reflects source and photo archive
tract_photo_data$tract_photo <- paste("",format(as.Date(tract_photo_data$FormToday, format="%d-%b-%Y"),"%Y"),"Core","Field","Tract",tract_photo_data$revision,tract_photo_data$tract_photo,sep="/")


# All tracts with id nos. under 1000 should be removed from analysis as they are for testing only
cat("\n Excluded because TractNo< 1000 i.e. these are tests and not real survey data \n \n ")

tract_photo_data[tract_photo_data$TractNo < 1000]

tract_photo_data <- tract_photo_data[tract_photo_data$TractNo > 999,]

# Order by TractNo
tract_photo_data <- tract_photo_data[order(tract_photo_data$TractNo), ]
rownames(tract_photo_data) <- 1:nrow(tract_photo_data)

### Rename columns according to description variables
# Change variable names with file description
tract_photo_data <- subset(tract_photo_data,
                     select = as.character(description_tract_photo_data$ODKExportName))
colnames(tract_photo_data) <- description_tract_photo_data$Variable
tract_photo_data <- tract_photo_data[!is.na(colnames(tract_photo_data))]

# Remove unwanted columns
tract_photo_data$`_NOT_EXPORTED_TO_CANONICAL_` <- NULL

# change na fields to empty quote
tract_photo_data[is.na(tract_photo_data)] <- ""

# Make a copy of tract_photo_data
tract_photo_data_test <- tract_photo_data

for (i in 1:ncol(tract_photo_data_test)) {
  if (is.factor(tract_photo_data_test[,i]) == TRUE) {
    tract_photo_data_test[,i] <- as.character(tract_photo_data_test[,i])
  } else
  if (is.integer(tract_photo_data_test[,i]) == TRUE) {
    tract_photo_data_test[,i] <- as.numeric(tract_photo_data_test[,i])
  }
}

# Read canonical data and make them similar to tract data
tract_photo_data_initial_path <- paste(wd_survey, tract_folder_canonical, tract_canonical_photo, sep = "")
if (file.exists(tract_photo_data_initial_path)) {
  tract_photo_data_initial <- read.csv(tract_photo_data_initial_path, stringsAsFactors = FALSE)
  for (i in 1:ncol(tract_photo_data_initial)) {
    if (is.integer(tract_photo_data_initial[,i]) == TRUE) {
      tract_photo_data_initial[,i] <- as.numeric(tract_photo_data_initial[,i])
    }
  }

  test_diff <- identical(tract_photo_data_test, tract_photo_data_initial)
  rm(tract_photo_data_test, tract_photo_data_initial)



  # Exporting data if needed and user say Y
  if (test_diff == FALSE) {
    # while (test_diff != "Y" | test_diff != "N") {
    #   cat("\n \n The 'Tract_photo_data' file has been changed and")
    #   cat("\n the canonical version will be updated.")
    #   cat("\n Do you agree [Y/N]? ")
    #   cat("\n (use a capital letter)   :")
    #   test_diff  <- scan("stdin", character(), n = 1, quiet = TRUE)
    #   if (test_diff == "Y") {
        write.csv(tract_photo_data,
                  file = paste(wd_survey,tract_folder_canonical,
                               tract_canonical_photo, sep = ""),
                  row.names = FALSE)
        cat("\n Canonical data has been overwritten\n \n")
      #   break}  # breaks when hit enter
      # if (test_diff == "N") {
      #   cat("\n Canonical data has NOT been overwritten\n \n")
      #   break}  # breaks when hit enter
    # }
  } else
    cat("\n tract_photo_data are already identical to canonical data \n \n ")
}
if (!file.exists(tract_photo_data_initial_path)) {
  write.csv(tract_photo_data,
          file = paste(wd_survey,tract_folder_canonical,
                        tract_canonical_photo, sep = ""),
          row.names = FALSE)
  cat("\n a file Tract_photos.csv has been added in the folder canonical \n \n ")
}
